/* BDF - Copyright (c) 2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstruction;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.session.LoginException;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.multi.MultiConf;
import fr.exemole.bdfserver.multi.MultiConstants;
import fr.exemole.bdfserver.multi.MultiResponseHandlerFactory;
import fr.exemole.bdfserver.tools.apps.AppConf;
import fr.exemole.bdfserver.tools.apps.AppConfUtils;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.OutputInfo;
import net.mapeadores.util.servlets.HttpAccessException;
import net.mapeadores.util.servlets.HttpServletRequestMap;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public final class BDFRoutes {

    private BDFRoutes() {

    }

    public static void resolveMulti(BDFWebapps bdfWebapps, String pathInfo, HttpServletRequestMap requestMap, HttpServletResponse response) throws IOException, ServletException {
        int idx = pathInfo.indexOf('/');
        if (idx == -1) {
            int idx2 = pathInfo.indexOf('-');
            if (idx2 == -1) {
                switch (pathInfo) {
                    case "_admin":
                        OutputInfo outputInfo = OutputInfo.buildFromRequest(requestMap, null);
                        if (outputInfo.getType() == OutputInfo.UNKNOWN_TYPE) {
                            BDFUtils.sendRedirect(response, "admin-multi");
                        } else {
                            resolveAdminMulti(bdfWebapps, requestMap, response, outputInfo);
                        }
                        break;
                    default:
                        BdfServer bdfServer = bdfWebapps.getBdfServer(pathInfo);
                        if (bdfServer != null) {
                            BDFUtils.sendRedirect(response, pathInfo + "/" + Domains.SESSION);
                        } else {
                            throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                        }
                }
            } else {
                String firstPart = pathInfo.substring(0, idx2);
                String secondPart = pathInfo.substring(idx2 + 1);
                switch (firstPart) {
                    case "admin":
                        OutputInfo outputInfo = OutputInfo.buildFromRequest(requestMap, secondPart);
                        resolveAdminMulti(bdfWebapps, requestMap, response, outputInfo);
                        break;
                    case "app":
                        AppConf appConf = AppConfUtils.getAppConf(bdfWebapps.getWebappsResourceStorages(), secondPart);
                        if ((appConf != null) && (appConf.getBoolean(AppConf.MULTI))) {
                            ResponseHandler responseHandler = MultiResponseHandlerFactory.getAppHtmlRespondeHandler(bdfWebapps, appConf);
                            responseHandler.handleResponse(response);
                        } else {
                            throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                        }
                        break;
                    default:
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                }
            }
        } else {
            String firstPart = pathInfo.substring(0, idx);
            String secondPart = pathInfo.substring(idx + 1);
            switch (firstPart) {
                case "_rsc": {
                    ResponseHandler responseHandler = MultiResponseHandlerFactory.getResourceResponseHandler(bdfWebapps, secondPart);
                    if (responseHandler != null) {
                        responseHandler.handleResponse(response);
                    } else {
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                    }
                    break;
                }
                default: {
                    BdfServer bdfServer = bdfWebapps.getBdfServer(firstPart);
                    if (bdfServer == null) {
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                    }
                    if (secondPart.isEmpty()) {
                        BDFUtils.sendRedirect(response, Domains.SESSION);
                    } else {
                        BDFRoutes.resolveBdfServer(bdfWebapps, bdfServer, secondPart, requestMap, response);
                    }
                }
            }
        }
    }

    public static void resolveBdfServer(BDFWebapps bdfWebapps, BdfServer bdfServer, String pathInfo, HttpServletRequestMap requestMap, HttpServletResponse response) throws IOException, ServletException {
        HttpServletRequest request = requestMap.getRequest();
        if (bdfServer.getInitState() == BdfServer.NOADMIN_STATE) {
            HttpAccessException exception = new HttpAccessException(HttpAccessException.INTERNAL_ERROR, "No Admin State", LogUtils.error("_ error.unsupported.noadminstate"));
            BDFUtils.sendError(request, response, exception, requestMap, bdfServer, null);
            return;
        }
        BdfInstruction bdfInstruction = BDFUtils.getBdfInstruction(bdfServer, requestMap, pathInfo);
        if (bdfInstruction == null) {
            HttpAccessException exception = new HttpAccessException(HttpAccessException.NOT_FOUND);
            BDFUtils.sendError(request, response, exception, requestMap, bdfServer, null);
            return;
        }
        BdfUser bdfUser = null;
        short needBdfUser = bdfInstruction.getBdfUserNeed();
        if (needBdfUser != BdfInstructionConstants.NONE_BDFUSER) {
            try {
                bdfUser = bdfWebapps.getBdfUser(bdfServer, requestMap, request);
                if (needBdfUser == BdfInstructionConstants.EXIT_BDFUSER) {
                    bdfWebapps.invalidateBdfUser(bdfServer, bdfUser);
                    throw new LoginException(LoginException.EXIT_ERROR);
                }
            } catch (LoginException le) {
                if (needBdfUser != BdfInstructionConstants.IF_ANY_BDFUSER) {
                    ResponseHandler responseHandler = LoginEngine.resolveLoginException(bdfServer, requestMap, pathInfo, bdfInstruction, le);
                    responseHandler.handleResponse(response);
                    return;
                }
            }
        }
        try {
            ResponseHandler responseHandler = bdfInstruction.runInstruction(bdfUser);
            if (responseHandler == null) {
                throw new HttpAccessException(HttpAccessException.NOT_FOUND);
            }
            long lastModified = responseHandler.getLastModified();
            if (lastModified == ResponseHandler.LASTMODIFIED_NOCACHE) {
                response.setHeader("Cache-Control", "no-store");
            } else if (lastModified > 0) {
                response.setDateHeader("Last-Modified", lastModified);
                try {
                    long since = request.getDateHeader("If-Modified-Since");
                    if (since >= lastModified) {
                        response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
                        return;
                    }
                } catch (IllegalArgumentException iae) {
                }
            }
            responseHandler.handleResponse(response);
        } catch (HttpAccessException hae) {
            BDFUtils.sendError(request, response, hae, requestMap, bdfServer, bdfUser);
        }
    }

    private static void resolveAdminMulti(BDFWebapps bdfWebapps, HttpServletRequestMap requestMap, HttpServletResponse response, OutputInfo outputInfo) throws IOException, ServletException {
        MultiConf multiConf = bdfWebapps.getMultiConf();
        if (outputInfo.getType() == OutputInfo.PAGE_TYPE) {
            multiConf.update();
            if (requestMap.isTrue(MultiConstants.RELOAD_PARAMNAME)) {
                bdfWebapps.getWebappsJsAnalyser().clearCache();
            }
        }
        ResponseHandler responseHandler;
        CommandMessage initErrorMessage = multiConf.getInitErrorMessage();
        if (initErrorMessage != null) {
            if (outputInfo.getType() == OutputInfo.JSON_TYPE) {
                responseHandler = CommandMessageResponseHandler.getJsonInstance(multiConf.getMessageLocalisation(), requestMap, initErrorMessage);
            } else if (initErrorMessage.getMessageKey().equals(MultiConf.MISSING_FILE_ERROR)) {
                responseHandler = MultiResponseHandlerFactory.getIniResponseHandler(bdfWebapps, requestMap);
            } else {
                responseHandler = CommandMessageResponseHandler.getHtmlInstance(multiConf.getMessageLocalisation(), initErrorMessage);
            }
        } else {
            switch (outputInfo.getType()) {
                case OutputInfo.JSON_TYPE:
                    short check = bdfWebapps.checkMultiAdmin(requestMap);
                    responseHandler = MultiResponseHandlerFactory.getJsonRespondeHandler(bdfWebapps, check, requestMap, outputInfo.getOutput());
                    break;
                case OutputInfo.STREAM_TYPE:
                    responseHandler = MultiResponseHandlerFactory.getStreamRespondeHandler(bdfWebapps, outputInfo.getOutput());
                    break;
                default:
                    responseHandler = MultiResponseHandlerFactory.getAdminHtmlRespondeHandler(bdfWebapps);
            }
        }
        responseHandler.handleResponse(response);
    }

}
