/* BDF - Copyright (c) 2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstruction;
import fr.exemole.bdfserver.api.interaction.Domain;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.get.GetInstructionFactory;
import fr.exemole.bdfserver.servlets.instructions.AppInstruction;
import fr.exemole.bdfserver.servlets.instructions.DomainInstruction;
import fr.exemole.bdfserver.servlets.instructions.RunInstruction;
import fr.exemole.bdfserver.servlets.instructions.SessionInstruction;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.io.IOException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.localisation.LangPreference;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.HttpAccessException;


/**
 *
 * @author Vincent Calame
 */
public final class BDFUtils {

    private BDFUtils() {

    }

    public static Domain parsePathInfo(String pathInfo) {
        String firstPart = pathInfo.toLowerCase();
        String secondPart = "";
        int length = pathInfo.length();
        for (int i = 0; i < length; i++) {
            char carac = pathInfo.charAt(i);
            boolean isSeparator = false;
            switch (carac) {
                case ':':
                case '_':
                case '-':
                    isSeparator = true;
                    break;
            }
            if (isSeparator) {
                secondPart = firstPart.substring(i + 1);
                firstPart = firstPart.substring(0, i);
                break;
            }
        }
        return new Domain(firstPart, secondPart);
    }

    public static BdfInstruction getDomainBdfInstruction(BdfServer bdfServer, RequestMap requestMap, Domain domain) {
        switch (domain.getFirstPart()) {
            case Domains.SESSION:
                return new SessionInstruction(bdfServer, requestMap);
            case Domains.RUN:
                return new RunInstruction(bdfServer, requestMap);
            case Domains.APP:
                return AppInstruction.build(bdfServer, requestMap, domain.getSecondPart());
            case Domains.ADMINISTRATION:
            case Domains.ALBUM:
            case Domains.CONFIGURATION:
            case Domains.CORPUS:
            case Domains.MAILING:
            case Domains.EXPORTATION:
            case Domains.EDITION:
            case Domains.SELECTION:
            case Domains.SPHERE:
            case Domains.THESAURUS:
            case Domains.MAIN:
            case Domains.MISC:
            case Domains.ADDENDA:
            case Domains.IMPORTATION:
            case Domains.PIOCHE:
            case Domains.EXTENSION:
                return new DomainInstruction(bdfServer, requestMap, domain);
            default:
                return null;
        }
    }

    public static int toHttpStatus(HttpAccessException hae) {
        switch (hae.getHttpType()) {
            case HttpAccessException.BAD_REQUEST:
                return HttpServletResponse.SC_BAD_REQUEST;
            case HttpAccessException.NOT_FOUND:
                return HttpServletResponse.SC_NOT_FOUND;
            case HttpAccessException.FORBIDDEN_ACCESS:
                return HttpServletResponse.SC_FORBIDDEN;
            case HttpAccessException.INTERNAL_ERROR:
                return HttpServletResponse.SC_INTERNAL_SERVER_ERROR;
            default:
                return HttpServletResponse.SC_INTERNAL_SERVER_ERROR;
        }
    }

    public static BdfInstruction getBdfInstruction(BdfServer bdfServer, RequestMap requestMap, String pathInfo) {
        int idx = pathInfo.indexOf('/');
        if (idx == -1) {
            Domain domain = BDFUtils.parsePathInfo(pathInfo);
            return BDFUtils.getDomainBdfInstruction(bdfServer, requestMap, domain);
        } else {
            String first = pathInfo.substring(0, idx);
            String next = pathInfo.substring(idx + 1);
            return GetInstructionFactory.getBdfInstruction(bdfServer, requestMap, first, next);
        }
    }

    public static void sendError(HttpServletRequest request, HttpServletResponse response, HttpAccessException hae) throws IOException {
        sendError(request, response, hae, null, null, null);
    }

    public static void sendError(HttpServletRequest request, HttpServletResponse response, HttpAccessException hae, RequestMap requestMap, BdfServer bdfServer, BdfUser bdfUser) throws IOException {
        CommandMessage commandMessage = hae.getCommandMessage();
        String message = null;
        if ((commandMessage != null) && (bdfServer != null)) {
            LangPreference langPreference;
            if (bdfUser == null) {
                langPreference = BdfInstructionUtils.getLangPreference(requestMap, bdfServer);
            } else {
                langPreference = bdfUser.getLangPreference();
            }
            message = bdfServer.getL10nManager().getMessageLocalisation(langPreference).toString(commandMessage);
        }
        if (message == null) {
            message = hae.getMessage();
        }
        if (message == null) {
            message = request.getServletPath() + request.getPathInfo();
        }
        response.sendError(BDFUtils.toHttpStatus(hae), message);
    }


    public static void sendRedirect(HttpServletResponse response, String path) throws IOException {
        response.sendRedirect(path);
    }

}
