/* BdfServer - Copyright (c) 2019 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.exportation.balayage;

import fr.exemole.bdfserver.api.balayage.Balayage;
import fr.exemole.bdfserver.api.storage.BalayageStorage;
import fr.exemole.bdfserver.api.storage.BdfStorageException;
import fr.exemole.bdfserver.api.storage.StorageContent;
import fr.exemole.bdfserver.tools.balayage.BalayageBuilder;
import fr.exemole.bdfserver.tools.balayage.dom.BalayageDOMReader;
import java.io.IOException;
import java.io.InputStream;
import net.fichotheque.Fichotheque;
import net.fichotheque.exportation.balayage.BalayageDescription;
import net.fichotheque.tools.exportation.balayage.BalayageContentDescriptionBuilder;
import net.fichotheque.tools.exportation.balayage.BalayageDescriptionBuilder;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class BalayageCompiler {

    private final static String SEVERE_XML = "severe.xml";
    private final Fichotheque fichotheque;

    public BalayageCompiler(Fichotheque fichotheque) {
        this.fichotheque = fichotheque;
    }

    public Result compile(BalayageStorage.Unit storageUnit) {
        String balayageName = storageUnit.getName();
        BalayageBuilder balayageBuilder = new BalayageBuilder(storageUnit.getName());
        BalayageDescriptionBuilder balayageDescriptionBuilder = new BalayageDescriptionBuilder(balayageName);
        BalayageDOMReader balayageReader = new BalayageDOMReader(fichotheque, balayageBuilder);
        StorageContent coreContent = storageUnit.getStorageContent("balayage.xml");
        boolean done = false;
        if (coreContent != null) {
            BalayageContentDescriptionBuilder builder = new BalayageContentDescriptionBuilder(balayageName, coreContent.getPath());
            try (InputStream is = coreContent.getInputStream()) {
                balayageReader.readBalayage(DOMUtils.parseDocument(is).getDocumentElement());
                done = true;
            } catch (SAXException saxe) {
                LogUtils.handleSAXException(SEVERE_XML, saxe, builder.getLineMessageHandler());
            } catch (IOException ioe) {
                throw new BdfStorageException(ioe);
            }
            balayageDescriptionBuilder.addBalayageContentDescription(builder.toBalayageContentDescription());
        }

        StorageContent includeCatalog = storageUnit.getStorageContent("include-catalog.xml");
        if (includeCatalog != null) {
            BalayageContentDescriptionBuilder builder = new BalayageContentDescriptionBuilder(balayageName, includeCatalog.getPath());
            try (InputStream is = includeCatalog.getInputStream()) {
                Document document = DOMUtils.parseDocument(is);
                if (done) {
                    balayageReader.readIncludeCatalog(document.getDocumentElement());
                }
            } catch (SAXException saxe) {
                LogUtils.handleSAXException(SEVERE_XML, saxe, builder.getLineMessageHandler());
            } catch (IOException ioe) {
                throw new BdfStorageException(ioe);
            }
            balayageDescriptionBuilder.addBalayageContentDescription(builder.toBalayageContentDescription());
        }
        for (StorageContent storageContent : storageUnit.getStorageContentList()) {
            BalayageContentDescriptionBuilder builder = new BalayageContentDescriptionBuilder(balayageName, storageContent.getPath());
            try (InputStream is = storageContent.getInputStream()) {
                DOMUtils.parseDocument(is);
            } catch (SAXException saxe) {
                LogUtils.handleSAXException(SEVERE_XML, saxe, builder.getLineMessageHandler());
            } catch (IOException ioe) {
                throw new BdfStorageException(ioe);
            }
            balayageDescriptionBuilder.addBalayageContentDescription(builder.toBalayageContentDescription());
        }
        Balayage balayage = null;
        if (done) {
            balayage = balayageBuilder.toBalayage();
        }
        balayageDescriptionBuilder.setStateEnabled((balayage != null));
        return new Result(balayageDescriptionBuilder.toBalayageDescription(), balayage);
    }


    public static class Result {

        private final BalayageDescription balayageDescription;
        private final Balayage balayage;

        private Result(BalayageDescription balayageDescription, Balayage balayage) {
            this.balayageDescription = balayageDescription;
            this.balayage = balayage;
        }

        public BalayageDescription getBalayageDescription() {
            return balayageDescription;
        }

        public Balayage getBalayage() {
            return balayage;
        }

    }

}
