/* BdfServer - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.runners;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.api.exportation.sql.BdfServerSqlExport;
import fr.exemole.bdfserver.api.externalscript.ExternalScript;
import fr.exemole.bdfserver.api.instruction.BdfParameters;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.configuration.ConfigurationUtils;
import fr.exemole.bdfserver.tools.configuration.Target;
import fr.exemole.bdfserver.tools.exportation.table.BdfTableExportUtils;
import fr.exemole.bdfserver.tools.exportation.table.CellEngineUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;
import java.util.function.Predicate;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.exportation.sql.SqlExport;
import net.fichotheque.exportation.sql.SqlExportDef;
import net.fichotheque.exportation.table.CellEngine;
import net.fichotheque.exportation.table.TableExport;
import net.fichotheque.exportation.table.TableExportContext;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FichothequeQuery;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.tools.exportation.sql.DefaultSqlExport;
import net.fichotheque.utils.EligibilityUtils;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.utils.selection.MotcleSelectorBuilder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;


/**
 *
 * @author Vincent Calame
 */
public final class SqlExportRunner {

    private SqlExportRunner() {

    }

    public static CommandMessage run(SqlExportDef sqlExportDef, BdfServer bdfServer, PathConfiguration pathConfiguration, boolean runExternalScript) throws IOException {
        BdfParameters firstAdminParameters = BdfUserUtils.getFirstAdminBdfParameters(bdfServer);
        PermissionSummary permissionSummary = firstAdminParameters.getPermissionSummary();
        String className = sqlExportDef.getSqlExportClassName();
        SqlExport sqlExport;
        if (className.isEmpty()) {
            sqlExport = new DefaultSqlExport();
        } else {
            try {
                Class classe = Class.forName(className);
                Object obj = classe.newInstance();
                sqlExport = (SqlExport) obj;
            } catch (ClassNotFoundException cnfe) {
                return LogUtils.error("_ error.exception.classnotfound", className);
            } catch (IllegalAccessException iae) {
                return LogUtils.error("_ error.exception.illegalaccess", className);
            } catch (InstantiationException ie) {
                return LogUtils.error("_ error.exception.instantiation", className);
            } catch (ClassCastException cce) {
                return LogUtils.error("_ error.exception.classcast", className, "SqlExport");
            }
        }
        FichothequeQuery fichothequeQuery = sqlExportDef.getCustomFichothequeQuery();
        List<FicheQuery> ficheQueryList = fichothequeQuery.getFicheQueryList();
        Lang defaultLang = ConfigurationUtils.getDefaultWorkingLang(bdfServer.getLangConfiguration());
        Predicate<FicheMeta> fichePredicate = null;
        if (!ficheQueryList.isEmpty()) {
            SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfServer, defaultLang)
                    .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                    .toSelectionContext();
            FicheSelectorBuilder ficheSelectorBuilder = FicheSelectorBuilder.init(selectionContext);
            for (FicheQuery ficheQuery : ficheQueryList) {
                ficheSelectorBuilder.add(ficheQuery, null);
            }
            fichePredicate = SelectionUtils.toPredicate(ficheSelectorBuilder.toFicheSelector());
        }
        List<MotcleQuery> motcleQueryList = fichothequeQuery.getMotcleQueryList();
        Predicate<Motcle> motclePredicate = null;
        if (!motcleQueryList.isEmpty()) {
            SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfServer, defaultLang)
                    .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                    .setFichePredicate(fichePredicate)
                    .toSelectionContext();
            MotcleSelectorBuilder motcleSelectorBuilder = MotcleSelectorBuilder.init(selectionContext);
            for (MotcleQuery motcleQuery : motcleQueryList) {
                motcleSelectorBuilder.add(motcleQuery, null);
            }
            motclePredicate = SelectionUtils.toPredicate(motcleSelectorBuilder.toMotcleSelector());
        }
        TableExport tableExport = null;
        String tableExportName = sqlExportDef.getTableExportName();
        if (!tableExportName.isEmpty()) {
            tableExport = bdfServer.getTableExportManager().getTableExport(tableExportName);
        }
        LangContext langContext = null;
        if (tableExport == null) {
            tableExport = BdfTableExportUtils.toDefaultTableExport(bdfServer, BdfTableExportUtils.NONE_FICHETABLEPARAMETERS, permissionSummary);
        } else {
            langContext = BdfServerUtils.checkLangContext(bdfServer, tableExport.getTableExportDef());
        }
        if (langContext == null) {
            langContext = ConfigurationUtils.toDefaultLangContext(bdfServer.getLangConfiguration());
        }
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, langContext)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toExtractionContext();
        CellEngine cellEngine = CellEngineUtils.newCellEngine(bdfServer, extractionContext, tableExport, true);
        sqlExport.setExtractionContext(extractionContext);
        TableExportContext tableExportContext = bdfServer.getTableExportContext();
        sqlExport.setTableExport(tableExport, tableExportContext);
        sqlExport.setCellEngine(cellEngine);
        sqlExport.setPredicate(EligibilityUtils.merge(fichePredicate, motclePredicate));
        for (SqlExportDef.Param param : sqlExportDef.getParamList()) {
            sqlExport.setParameter(param.getName(), param.getValue());
        }
        Target target = ConfigurationUtils.getTarget(pathConfiguration, sqlExportDef);
        if (sqlExport instanceof BdfServerSqlExport) {
            ((BdfServerSqlExport) sqlExport).setBdfServer(bdfServer);
        }
        try (OutputStream os = new FileOutputStream(target.getFile())) {
            sqlExport.exportDump(os);
        }
        if (runExternalScript) {
            ExternalScript externalScript = bdfServer.getExternalScriptManager().getExternalScript(sqlExportDef.getPostCommand());
            if (externalScript != null) {
                try {
                    externalScript.exec();
                } catch (IOException ioe) {
                }
            }
        }
        return LogUtils.done("_ done.exportation.sqlexportrun");
    }

}
