/* BdfServer_Commands - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.administration;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.managers.PermissionManager;
import fr.exemole.bdfserver.api.roles.FichePermission;
import fr.exemole.bdfserver.api.roles.Permission;
import fr.exemole.bdfserver.api.roles.Role;
import fr.exemole.bdfserver.api.roles.RoleEditor;
import fr.exemole.bdfserver.commands.BdfCommandUtils;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.roles.RoleUtils;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.tools.parsers.AttributeParser;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.LabelChange;


/**
 *
 * @author Vincent Calame
 */
public class RoleChangeCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "RoleChange";
    public final static String COMMANDKEY = "_ ROL-02";
    public final static String PERMISSION_PARAMNAME = "permission";
    public final static String LEVEL_PARAMSUFFIX = "|level";
    public final static String CREATEFICHE_PARAMSUFFIX = "|createfiche";
    public final static String READFICHE_PARAMSUFFIX = "|readfiche";
    public final static String WRITEFICHE_PARAMSUFFIX = "|writefiche";
    public final static String TITLE_PARAMPREFIX = "title/";
    public final static String ATTRIBUTES_PARAMNAME = "attributes";
    private Role role;
    private LabelChange labelChange;
    private AttributeChange attributeChange;
    private final Map<SubsetKey, Permission> permissionMap = new HashMap<SubsetKey, Permission>();

    public RoleChangeCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        BdfServerEditor bdfServerEditor = bdfServer.getBdfServerEditor();
        RoleEditor roleEditor = bdfServerEditor.getRoleEditor();
        BdfServerUtils.changeLabels(bdfServerEditor, role, labelChange);
        if (attributeChange != null) {
            BdfServerUtils.changeAttributes(bdfServerEditor, role, attributeChange);
        }
        Subset[] subsetArray = FichothequeUtils.toSubsetArray(fichotheque);
        for (Subset subset : subsetArray) {
            SubsetKey subsetKey = subset.getSubsetKey();
            Permission permission = permissionMap.get(subsetKey);
            if (permission != null) {
                roleEditor.putSubsetPermission(role, subsetKey, permission);
            } else {
                roleEditor.removeSubsetPermission(role, subsetKey);
            }
        }
        bdfServerEditor.saveChanges();
        setDone("_ done.administration.rolechange");
        putResultObject(ROLE_OBJ, role);
    }


    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkGlobalAdmin();
        PermissionManager permissionManager = bdfServer.getPermissionManager();
        role = BdfInstructionUtils.getMandatoryRole(permissionManager, requestMap);
        labelChange = BdfCommandUtils.getLabelChange(requestMap, TITLE_PARAMPREFIX, true);
        String[] permissionArray = requestMap.getParameterValues(PERMISSION_PARAMNAME);
        if (permissionArray != null) {
            for (String permission : permissionArray) {
                try {
                    SubsetKey subsetKey = SubsetKey.parse(permission);
                    initSubsetPermission(subsetKey);
                } catch (ParseException pe) {
                    throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PERMISSION_PARAMNAME, permission));
                }
            }
        }
        String attributes = requestMap.getParameter(ATTRIBUTES_PARAMNAME);
        if (attributes != null) {
            attributeChange = AttributeParser.parse(attributes);
        }
        return true;
    }

    private void initSubsetPermission(SubsetKey subsetKey) throws BdfInstructionException {
        String levelParamName = subsetKey.toString() + LEVEL_PARAMSUFFIX;
        String levelString = getMandatory(levelParamName);
        short level;
        Permission.CustomPermission customPermission = null;
        try {
            level = RoleUtils.levelToShort(levelString);
        } catch (IllegalArgumentException iae) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(levelParamName, levelString));
        }
        if ((level == Permission.CUSTOM_LEVEL) && (subsetKey.isCorpusSubset())) {
            boolean create = requestMap.isTrue(subsetKey.toString() + CREATEFICHE_PARAMSUFFIX);
            short read = getType(subsetKey.toString() + READFICHE_PARAMSUFFIX);
            short write = getType(subsetKey.toString() + WRITEFICHE_PARAMSUFFIX);
            FichePermission fichePermission = RoleUtils.toFichePermission(create, read, write);
            if (RoleUtils.matchStandardPermission(fichePermission)) {
                level = Permission.STANDARD_LEVEL;
            } else {
                customPermission = fichePermission;
            }
        }
        permissionMap.put(subsetKey, RoleUtils.toPermission(level, customPermission));
    }

    private short getType(String paramName) throws BdfInstructionException {
        String fichePermissionString = requestMap.getParameter(paramName);
        if (fichePermissionString == null) {
            throw new BdfInstructionException(BdfInstructionUtils.emptyMandatoryParameter(paramName));
        }
        try {
            short type = RoleUtils.fichePermissionTypeToShort(fichePermissionString);
            return type;
        } catch (IllegalArgumentException iae) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(paramName, fichePermissionString));
        }
    }

}
