/* BdfServer_Commands - Copyright (c) 2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.configuration;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.storage.IconScanEngine;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import java.io.IOException;
import java.io.InputStream;
import net.fichotheque.EditOrigin;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.request.FileValue;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public class CustomIconUploadCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "LogoUpload";
    public final static String COMMANDKEY = "_ CNF-09";
    public final static String FILE_PARAMNAME = "file";
    public final static String REL_PARAMNAME = "rel";
    public final static String SIZES_PARAMNAME = "sizes";
    public final static String ICON_REL_PARAMVALUE = "icon";
    public final static String TOUCH_REL_PARAMVALUE = "touch";
    public final static String SHORTCUT_REL_PARAMVALUE = "shortcut";
    private String basename;

    public CustomIconUploadCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        FileValue fileValue = requestMap.getFileValue(FILE_PARAMNAME);
        if (fileValue == null) {
            throw new BdfInstructionException(BdfInstructionUtils.emptyMandatoryParameter(FILE_PARAMNAME));
        }
        String resourceName = checkExtension(fileValue.getName());
        if (resourceName == null) {
            fileValue.free();
            setError("_ error.empty.extension", fileValue.getName());
            return;
        }
        RelativePath relativePath = IconScanEngine.ICON_FOLDER.buildChild(resourceName);
        boolean done = false;
        if (fileValue.length() > 0) {
            EditOrigin editOrigin = bdfUser.newEditOrigin(Domains.CONFIGURATION + "/" + COMMANDNAME);
            try (InputStream is = fileValue.getInputStream()) {
                StorageUtils.saveResource(bdfServer, relativePath, is, editOrigin);
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
            done = true;
        }
        fileValue.free();
        if (done) {
            putResultObject(RELATIVEPATH_OBJ, relativePath);
            setDone("_ done.administration.customiconupload");
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkGlobalAdmin();
        String prefix = getPrefix(getMandatory(REL_PARAMNAME));
        String sizes = getMandatory(SIZES_PARAMNAME).trim();
        sizes = sizes.replace(" ", "_");
        String name = prefix + sizes;
        if (!StorageUtils.isValidResourceName(name + ".png")) {
            setError("_ error.wrong.sizes", sizes);
            return false;
        }
        this.basename = name;
        return true;
    }

    private String checkExtension(String fileName) {
        int idx = fileName.lastIndexOf(".");
        if (idx == -1) {
            return null;
        }
        String extension = fileName.substring(idx + 1).toLowerCase();
        if (extension.isEmpty()) {
            return null;
        }
        String result = basename + "." + extension;
        if (!StorageUtils.isValidResourceName(result)) {
            return null;
        }
        return result;
    }

    private static String getPrefix(String rel) throws BdfInstructionException {
        switch (rel) {
            case ICON_REL_PARAMVALUE:
                return IconScanEngine.ICON_PREFIX;
            case TOUCH_REL_PARAMVALUE:
                return IconScanEngine.APPLETOUCHICON_PREFIX;
            case SHORTCUT_REL_PARAMVALUE:
                return IconScanEngine.SHORTCUTICON_PREFIX;
            default:
                throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(REL_PARAMNAME, rel));

        }
    }


}
