/* BdfServer_Commands - Copyright (c) 2016-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.exportation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.managers.TableExportManager;
import fr.exemole.bdfserver.commands.BdfCommandUtils;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.text.ParseException;
import net.fichotheque.EditOrigin;
import net.fichotheque.exportation.table.TableExportDef;
import net.fichotheque.exportation.table.TableExportDescription;
import net.fichotheque.tools.exportation.table.TableExportDefBuilder;
import net.fichotheque.tools.parsers.AttributeParser;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelChange;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class TableExportDefChangeCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "TableExportDefChange";
    public final static String COMMANDKEY = "_ EXP-13";
    public final static String TITLE_PARAMPREFIX = "title/";
    public final static String ATTRIBUTES_PARAMNAME = "attributes";
    public final static String LANGMODE_PARAMNAME = "langmode";
    public final static String LANGLIST_PARAMNAME = "langlist";
    private TableExportDef tableExportDef;

    public TableExportDefChangeCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        EditOrigin editOrigin = bdfUser.newEditOrigin(Domains.EXPORTATION + "/" + COMMANDNAME);
        TableExportManager tableExportManager = bdfServer.getTableExportManager();
        TableExportDescription tableExportDescription = tableExportManager.putTableExportDef(tableExportDef, editOrigin);
        putResultObject(TABLEEXPORTDESCRIPTION_OBJ, tableExportDescription);
        setDone("_ done.exportation.tableexportdefchange", tableExportDef.getName());
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        checkGlobalAdmin();
        TableExportDescription tableExportDescription = BdfInstructionUtils.getMandatoryTableExportDescription(bdfServer.getTableExportManager(), requestMap);
        if (!tableExportDescription.isEditable()) {
            throw new BdfInstructionException(BdfInstructionUtils.unsupportedNotEditableParameterValue(InteractionConstants.TABLEEXPORT_PARAMNAME, tableExportDescription.getName()));
        }
        TableExportDefBuilder tableExportDefBuilder = new TableExportDefBuilder(tableExportDescription.getName(), tableExportDescription.getTableExportDef().getAttributes());
        LabelChange labelChange = BdfCommandUtils.getLabelChange(requestMap, TITLE_PARAMPREFIX, true);
        for (Label label : labelChange.getChangedLabels()) {
            tableExportDefBuilder.putLabel(label);
        }
        String attributes = requestMap.getParameter(ATTRIBUTES_PARAMNAME);
        if (attributes != null) {
            tableExportDefBuilder.getAttributesBuilder().changeAttributes(AttributeParser.parse(attributes));
        }
        String langMode = checkLangMode(requestMap.getParameter(LANGMODE_PARAMNAME));
        if ((langMode != null) && (langMode.equals(TableExportDescription.CUSTOMLIST_LANGMODE))) {
            String langListString = getMandatory(LANGLIST_PARAMNAME);
            String[] tokens = StringUtils.getTechnicalTokens(langListString, true);
            boolean done = false;
            for (String token : tokens) {
                try {
                    Lang lang = Lang.parse(token);
                    done = true;
                    tableExportDefBuilder.addLang(lang);
                } catch (ParseException mce) {
                }
            }
            if (!done) {
                langMode = null;
            }
        }
        tableExportDefBuilder.setLangMode(langMode);
        this.tableExportDef = tableExportDefBuilder.toTableExportDef();
        return true;
    }

    private static String checkLangMode(String langMode) {
        if (langMode == null) {
            return null;
        }
        switch (langMode) {
            case TableExportDescription.WORKING_LANGMODE:
            case TableExportDescription.SUPPLEMENTARY_LANGMODE:
            case TableExportDescription.CUSTOMLIST_LANGMODE:
                return langMode;
            default:
                return null;
        }
    }

}
