/* BdfServer_Commands - Copyright (c) 2011-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.mailing;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerConstants;
import fr.exemole.bdfserver.api.exportation.table.FicheTableParameters;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.MailingDomain;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.email.EmailBuffer;
import fr.exemole.bdfserver.email.FicheAttachmentParameters;
import fr.exemole.bdfserver.email.SendEngine;
import fr.exemole.bdfserver.email.SendReport;
import fr.exemole.bdfserver.email.SmtpManager;
import fr.exemole.bdfserver.email.TableExportEmail;
import fr.exemole.bdfserver.tools.exportation.table.TableExportParametersBuilder;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.interaction.FicheTableParametersParser;
import fr.exemole.bdfserver.tools.roles.PermissionUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.fichotheque.addenda.Version;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.utils.AddendaUtils;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.fichotheque.utils.TableExportUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class SendCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "Send";
    public final static String SELFBCC_PARAMNAME = "selfbcc";
    public final static String TYPE_PARAMNAME = "type";
    public final static String SUBJECT_PARAMNAME = "subject";
    public final static String TO_PARAMNAME = "to";
    public final static String CC_PARAMNAME = "cc";
    public final static String BCC_PARAMNAME = "bcc";
    public final static String MESSAGE_PARAMNAME = "message";
    public final static String DOCUMENTVERSION_PARAMNAME = "documentversion";
    public final static String FICHEVERSION_PARAMNAME = "ficheversion";
    public final static String HTML_FICHEVERSION_PARAMVALUE = "html";
    public final static String ODT_FICHEVERSION_PARAMVALUE = "odt";
    public final static String HTMLTEMPLATE_PARAMNAME = "htmltemplate";
    public final static String ODTTEMPLATE_PARAMNAME = "odttemplate";
    public final static String EXTENSION_PARAMNAME = "extension";
    private SmtpManager smtpManager;
    private EmailBuffer emailBuffer;

    public SendCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return false;
    }

    @Override
    protected void doCommand() throws BdfInstructionException {
        if (emailBuffer.hasWrongAddresses()) {
            setError("_ error.wrong.addresslist");
            return;
        }
        SendEngine sendEngine = new SendEngine(smtpManager, this);
        SendReport sendReport = sendEngine.sendEmail(emailBuffer);
        putResultObject(SENDREPORT_OBJ, sendReport);
        switch (sendReport.getState()) {
            case SendReport.OK_STATE:
                setDone(getDoneMessageKey());
                break;
            case SendReport.EMAILBUILD_ERROR_STATE:
                setError("_ error.exception.sendreport_emailbuild", sendReport.getErrorMessage());
                break;
            case SendReport.SEND_ERROR_STATE:
                setError("_ error.exception.sendreport_error", sendReport.getErrorMessage());
                break;
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        smtpManager = testSmtpManager(bdfServer);
        emailBuffer = buildFromRequest(bdfServer, bdfUser, getPermissionSummary(), requestMap, false);
        if (emailBuffer.getSubject() == null) {
            setError("_ error.empty.subject");
            return false;
        }
        if (emailBuffer.getMessage() == null) {
            setError("_ error.empty.message");
            return false;
        }
        if (emailBuffer.isWithoutField(EmailBuffer.TO_FIELD)) {
            setError("_ error.empty.to");
            return false;
        }
        putResultObject(EMAILBUFFER_OBJ, emailBuffer);
        return true;
    }

    private String getDoneMessageKey() {
        switch (emailBuffer.getSendType()) {
            case MailingDomain.FICHE_SENDTYPE:
                return "_ done.mailing.send_fiche";
            case MailingDomain.COMPILATION_SENDTYPE:
                return "_ done.mailing.send_compilation";
            case MailingDomain.SELECTION_SENDTYPE:
                return "_ done.mailing.send_selection";
            case MailingDomain.TABLEEXPORT_SENDTYPE:
                return "_ done.mailing.send_tableexport";
            default:
                throw new SwitchException("Unknown sendType: " + emailBuffer.getSendType());
        }
    }

    public static EmailBuffer buildFromRequest(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap, boolean withDefaultValue) throws BdfInstructionException {
        Redacteur redacteur = bdfUser.getRedacteur();
        String sendType = BdfInstructionUtils.getMandatoryParameter(requestMap, TYPE_PARAMNAME);
        EmailBuffer eb;
        switch (sendType) {
            case MailingDomain.FICHE_SENDTYPE:
                eb = initFiche(bdfServer, bdfUser, permissionSummary, requestMap, withDefaultValue);
                break;
            case MailingDomain.COMPILATION_SENDTYPE:
                eb = EmailBuffer.buildForCompilation(bdfServer, bdfUser);
                eb.setFicheAttachmentParameters(initFicheAttachmentParameters(bdfServer, bdfUser, requestMap, withDefaultValue, TransformationKey.COMPILATION_TRANSFORMATIONKEY));
                break;
            case MailingDomain.SELECTION_SENDTYPE:
                eb = initSelection(bdfServer, bdfUser, requestMap, withDefaultValue);
                break;
            case MailingDomain.TABLEEXPORT_SENDTYPE:
                eb = initTableExport(bdfServer, bdfUser, permissionSummary, requestMap, withDefaultValue);
                break;
            default:
                throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(TYPE_PARAMNAME, sendType));
        }
        parseAddresses(bdfServer, redacteur, requestMap, eb, EmailBuffer.TO_FIELD, TO_PARAMNAME);
        parseAddresses(bdfServer, redacteur, requestMap, eb, EmailBuffer.CC_FIELD, CC_PARAMNAME);
        parseAddresses(bdfServer, redacteur, requestMap, eb, EmailBuffer.BCC_FIELD, BCC_PARAMNAME);
        String subject = requestMap.getParameter(SUBJECT_PARAMNAME);
        if (subject != null) {
            eb.setSubject(subject);
        }
        boolean withBcc;
        if (withDefaultValue) {
            withBcc = true;
        } else {
            withBcc = requestMap.isTrue(SELFBCC_PARAMNAME);
        }
        eb.setWithRedacteurBcc(withBcc);
        eb.setMessage(requestMap.getParameter(MESSAGE_PARAMNAME));
        return eb;
    }

    private static void parseAddresses(BdfServer bdfServer, Redacteur redacteur, RequestMap requestMap, EmailBuffer eb, String fieldName, String paramName) {
        String[] values = requestMap.getParameterValues(paramName);
        if (values != null) {
            List<String> valueList = new ArrayList<String>();
            for (String toValue : values) {
                String[] tokens = StringUtils.getTechnicalTokens(toValue, false);
                for (String token : tokens) {
                    valueList.add(token);
                }
            }
            int size = valueList.size();
            if (size > 0) {
                eb.parseToArray(bdfServer.getFichotheque(), fieldName, valueList, redacteur.getSubsetKey());
            }
        }
    }

    private static EmailBuffer initFiche(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap, boolean withDefaultValue) throws BdfInstructionException {
        FicheMeta ficheMeta = BdfInstructionUtils.getMandatoryFicheMeta(bdfServer.getFichotheque(), requestMap);
        PermissionUtils.checkRead(permissionSummary, ficheMeta);
        EmailBuffer eb = EmailBuffer.buildForFiche(bdfServer, bdfUser, ficheMeta);
        if ((withDefaultValue) && (eb.getSubject() == null)) {
            eb.setSubject(CorpusMetadataUtils.getFicheTitle(ficheMeta, bdfUser.getWorkingLang(), bdfUser.getFormatLocale()));
        }
        eb.setFicheAttachmentParameters(initFicheAttachmentParameters(bdfServer, bdfUser, requestMap, withDefaultValue, new TransformationKey(ficheMeta.getSubsetKey())));
        return eb;
    }

    private static EmailBuffer initTableExport(BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap, boolean withDefaultValue) throws BdfInstructionException {
        Corpus corpus = BdfInstructionUtils.getMandatoryCorpus(bdfServer.getFichotheque(), requestMap);
        if (!permissionSummary.hasAccess(corpus)) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue("corpus", corpus.getSubsetName()));
        }
        TableExportParametersBuilder builder = new TableExportParametersBuilder();
        String tableExportName = requestMap.getParameter(InteractionConstants.TABLEEXPORT_PARAMNAME);
        builder.setTableExportName(tableExportName);
        if (tableExportName == null) {
            try {
                FicheTableParameters FicheTableParameters = FicheTableParametersParser.parse(requestMap);
                builder.setFicheTableParameters(FicheTableParameters);
            } catch (ErrorMessageException errorMessageException) {
                throw new BdfInstructionException(errorMessageException.getErrorMessage());
            }
        }
        String headerTypeValue = requestMap.getParameter(InteractionConstants.HEADERTYPE_PARAMNAME);
        if (headerTypeValue != null) {
            try {
                short headerType = TableExportUtils.headerTypeToShort(headerTypeValue);
                builder.setHeaderType(headerType);
            } catch (IllegalArgumentException iae) {
                throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(InteractionConstants.HEADERTYPE_PARAMNAME, headerTypeValue));
            }
        }
        String charset = requestMap.getParameter(InteractionConstants.CHARSET_PARAMNAME);
        if (charset == null) {
            charset = "UTF-8";
        }
        String tableExportExtension = checkTableExportExtension(requestMap.getParameter(EXTENSION_PARAMNAME));
        return EmailBuffer.buildForTableExport(bdfServer, bdfUser, new TableExportEmail(builder.toTableExportParameters(), corpus, tableExportExtension, charset));
    }

    private static EmailBuffer initSelection(BdfServer bdfServer, BdfUser bdfUser, RequestMap requestMap, boolean withDefaultValue) throws BdfInstructionException {
        Set<FicheMeta> ficheMetaSet = new LinkedHashSet<FicheMeta>();
        for (Corpus corpus : bdfServer.getFichotheque().getCorpusList()) {
            String[] param_selection = requestMap.getParameterValues("selection_" + corpus.getSubsetName());
            if (param_selection != null) {
                for (String token : param_selection) {
                    try {
                        int id = Integer.parseInt(token);
                        FicheMeta ficheMeta = corpus.getFicheMetaById(id);
                        if (ficheMeta != null) {
                            ficheMetaSet.add(ficheMeta);
                        }
                    } catch (NumberFormatException nfe) {
                    }
                }
            }
        }
        EmailBuffer eb = EmailBuffer.buildForSelection(bdfServer, bdfUser, ficheMetaSet);
        TransformationKey transformationKey;
        Fiches selectedFiches = bdfUser.getSelectedFiches();
        if (selectedFiches.getEntryList().size() == 1) {
            transformationKey = new TransformationKey(selectedFiches.getEntryList().get(0).getCorpus().getSubsetKey());
        } else {
            transformationKey = null;
        }
        eb.setFicheAttachmentParameters(initFicheAttachmentParameters(bdfServer, bdfUser, requestMap, withDefaultValue, transformationKey));
        return eb;
    }

    private static FicheAttachmentParameters initFicheAttachmentParameters(BdfServer bdfServer, BdfUser bdfUser, RequestMap requestMap, boolean withDefaultValue, TransformationKey transformationKey) {
        FicheAttachmentParameters ficheAttachmentParameters = new FicheAttachmentParameters();
        String[] values = requestMap.getParameterValues(FICHEVERSION_PARAMNAME);
        if (values != null) {
            for (String value : values) {
                if (value.equals(HTML_FICHEVERSION_PARAMVALUE)) {
                    ficheAttachmentParameters
                            .setWithHtml(true)
                            .setHtmlTemplateName(requestMap.getParameter(HTMLTEMPLATE_PARAMNAME));
                } else if (value.equals(ODT_FICHEVERSION_PARAMVALUE)) {
                    ficheAttachmentParameters
                            .setWithOdt(true)
                            .setOdtTemplateName(requestMap.getParameter(ODTTEMPLATE_PARAMNAME));
                }
            }
        } else if (withDefaultValue) {
            ficheAttachmentParameters.setWithHtml(true);
            if (transformationKey != null) {
                ficheAttachmentParameters
                        .setHtmlTemplateName(BdfUserUtils.getSimpleTemplateName(bdfUser, transformationKey))
                        .setOdtTemplateName(BdfUserUtils.getStreamTemplateName(bdfUser, transformationKey, "odt"));
            }
        }
        String[] versionArray = requestMap.getParameterValues(DOCUMENTVERSION_PARAMNAME);
        if (versionArray != null) {
            for (String versionKey : versionArray) {
                try {
                    Version version = AddendaUtils.parseVersionKey(versionKey, bdfServer.getFichotheque());
                    ficheAttachmentParameters.addVersion(version);
                } catch (ParseException pe) {

                }
            }
        }
        return ficheAttachmentParameters;
    }

    public static SmtpManager testSmtpManager(BdfServer bdfServer) throws BdfInstructionException {
        SmtpManager smtpManager = (SmtpManager) bdfServer.getContextObject(BdfServerConstants.SMTPMANAGER_CONTEXTOBJECT);
        if (!smtpManager.isInit()) {
            StringBuilder buf = new StringBuilder();
            buf.append("Smtp parameters are not initialized ");
            for (String error : smtpManager.getErrorList()) {
                buf.append(" / ");
                buf.append(error);
            }
            buf.append(")");
            throw new BdfInstructionException(buf.toString());
        }
        return smtpManager;
    }

    private static String checkTableExportExtension(String tableExportExtension) throws BdfInstructionException {
        if (tableExportExtension == null) {
            return "csv";
        }
        if (tableExportExtension.equals("csv")) {
            return "csv";
        }
        if (tableExportExtension.equals("ods")) {
            return "ods";
        }
        throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(EXTENSION_PARAMNAME, tableExportExtension));
    }

}
