/* BdfServer_HtmlProducers - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.pioche;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.PiocheDomain;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.tools.externalsource.ExternalSourceUtils;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.externalsource.ExternalSource;
import net.fichotheque.thesaurus.Thesaurus;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.CommandMessageBuilder;
import net.mapeadores.util.primitives.DegreDecimal;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class PiocheHtmlProducerFactory {

    private PiocheHtmlProducerFactory() {
    }

    public static HtmlProducer getHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        BdfServerHtmlProducer htmlProducer = getBdfHtmlProducer(parameters);
        if (htmlProducer != null) {
            htmlProducer.setBdfCommandResult(parameters.getBdfCommandResult());
        }
        return htmlProducer;
    }

    private static BdfServerHtmlProducer getBdfHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        String page = parameters.getOutput();
        RequestMap requestMap = parameters.getRequestMap();
        Fichotheque fichotheque = parameters.getFichotheque();
        switch (page) {
            case PiocheDomain.EXTERNALSOURCE_PAGE: {
                return new ExternalSourceHtmlProducer(parameters, getExternalSourceParameters(requestMap, parameters.getBdfServer()));
            }
            case PiocheDomain.DOCUMENT_PAGE: {
                return new DocumentPiocheHtmlProducer(parameters, getDocumentParameters(requestMap, fichotheque));
            }
            case PiocheDomain.GEOCODAGE_PAGE: {
                return new GeocodageHtmlProducer(parameters, getGeocodageParameters(requestMap));
            }
            case PiocheDomain.FICHE_PAGE: {
                return new FichePiocheHtmlProducer(parameters, getFicheParameters(requestMap, fichotheque));
            }
            case PiocheDomain.MOTCLE_PAGE: {
                return new MotclePiocheHtmlProducer(parameters, getMotcleParameters(requestMap, fichotheque));
            }
            case PiocheDomain.REDACTEUR_PAGE: {
                return new RedacteurPiocheHtmlProducer(parameters, getRedacteurParameters(requestMap, fichotheque));
            }
            case PiocheDomain.LANGUE_PAGE: {
                return new LanguePiocheHtmlProducer(parameters, getParameters(requestMap));
            }
            case PiocheDomain.PAYS_PAGE: {
                return new PaysPiocheHtmlProducer(parameters, getParameters(requestMap));
            }
            case PiocheDomain.FIELD_PAGE: {
                return new FieldPiocheHtmlProducer(parameters, getFieldParameters(requestMap));
            }
            default:
                return null;
        }
    }

    private static PiocheParameters getParameters(RequestMap requestMap) throws BdfInstructionException {
        String appelant = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.APPELANT_PARAMNAME);
        int limit = getLimit(requestMap);
        return PiocheParameters.init()
                .setAppelant(appelant)
                .setLimit(limit);
    }

    private static PiocheParameters getFieldParameters(RequestMap requestMap) throws BdfInstructionException {
        String appelant = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.APPELANT_PARAMNAME);
        int limit = getLimit(requestMap);
        return PiocheParameters.init()
                .setAppelant(appelant)
                .setLimit(limit)
                .putParameter(PiocheParameters.FIELDTYPE, requestMap.getParameter(PiocheDomain.PARAM_PARAMNAME));
    }

    private static PiocheParameters getGeocodageParameters(RequestMap requestMap) throws BdfInstructionException {
        String appelant = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.APPELANT_PARAMNAME);
        String latString = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.LAT_PARAMNAME).trim();
        String lonString = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.LON_PARAMNAME).trim();
        DegreDecimal latDecimal = null;
        DegreDecimal lonDecimal = null;
        if ((latString.length() > 0) && (lonString.length() > 0)) {
            CommandMessageBuilder commandMessageBuilder = new CommandMessageBuilder();
            try {
                latDecimal = DegreDecimal.newInstance(StringUtils.parseDecimal(latString));
            } catch (NumberFormatException nfe) {
                commandMessageBuilder.addMultiError("_ error.wrong.geopoint_lat", latString);
            }
            try {
                lonDecimal = DegreDecimal.newInstance(StringUtils.parseDecimal(lonString));
            } catch (NumberFormatException nfe) {
                commandMessageBuilder.addMultiError("_ error.wrong.geopoint_lon", lonString);
            }
            if (commandMessageBuilder.hasMultiError()) {
                CommandMessage errorMessage = commandMessageBuilder
                        .setError("_ error.wrong.geopoint")
                        .toCommandMessage();
                throw new BdfInstructionException(errorMessage);
            }
        }
        return PiocheParameters.init()
                .setAppelant(appelant)
                .putParameter(PiocheParameters.LAT, latDecimal)
                .putParameter(PiocheParameters.LON, lonDecimal)
                .putParameter(PiocheParameters.ADRESSFIELDS, requestMap.getParameter(PiocheDomain.PARAM_PARAMNAME));
    }

    private static PiocheParameters getFicheParameters(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        String appelant = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.APPELANT_PARAMNAME);
        int limit = getLimit(requestMap);
        return PiocheParameters.init()
                .setAppelant(appelant)
                .setLimit(limit)
                .setSubsetList(getSubsetList(requestMap, fichotheque, SubsetKey.CATEGORY_CORPUS))
                .putParameter(PiocheParameters.MOTCLESELECTOPTIONS, requestMap.getParameter(PiocheDomain.MOTCLESELECTOPTIONS_PARAMNAME));
    }

    private static PiocheParameters getMotcleParameters(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        String appelant = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.APPELANT_PARAMNAME);
        int limit = getLimit(requestMap);
        return PiocheParameters.init()
                .setAppelant(appelant)
                .setLimit(limit)
                .setSubsetList(getSubsetList(requestMap, fichotheque, SubsetKey.CATEGORY_THESAURUS))
                .putParameter(PiocheParameters.CROISEMENT, getCroisementSubsetKey(requestMap))
                .putParameter(PiocheParameters.WANTED, requestMap.getParameter(PiocheDomain.WANTED_PARAMNAME));
    }

    private static PiocheParameters getDocumentParameters(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        String appelant = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.APPELANT_PARAMNAME);
        int limit = getLimit(requestMap);
        return PiocheParameters.init()
                .setAppelant(appelant)
                .setLimit(limit)
                .setSubsetList(getSubsetList(requestMap, fichotheque, SubsetKey.CATEGORY_ADDENDA));
    }

    private static PiocheParameters getExternalSourceParameters(RequestMap requestMap, BdfServer bdfServer) throws BdfInstructionException {
        String appelant = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.APPELANT_PARAMNAME);
        int limit = getLimit(requestMap);
        List<Subset> thesaurusList = getSubsetList(requestMap, bdfServer.getFichotheque(), SubsetKey.CATEGORY_THESAURUS);
        if (thesaurusList == null) {
            throw new BdfInstructionException(BdfInstructionUtils.emptyMandatoryParameter(PiocheDomain.SUBSETS_PARAMNAME));
        }
        ExternalSource externalSource = ExternalSourceUtils.getExternalSource(bdfServer, (Thesaurus) thesaurusList.get(0));
        if (externalSource == null) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, requestMap.getParameter(PiocheDomain.SUBSETS_PARAMNAME)));
        }
        return PiocheParameters.init()
                .setAppelant(appelant)
                .setLimit(limit)
                .setSubsetList(thesaurusList)
                .putParameter(PiocheParameters.EXTERNALSOURCE, externalSource)
                .putParameter(PiocheParameters.CROISEMENT, getCroisementSubsetKey(requestMap));
    }

    private static PiocheParameters getRedacteurParameters(RequestMap requestMap, Fichotheque fichotheque) throws BdfInstructionException {
        String appelant = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.APPELANT_PARAMNAME);
        int limit = getLimit(requestMap);
        return PiocheParameters.init()
                .setAppelant(appelant)
                .setLimit(limit)
                .setSubsetList(getSubsetList(requestMap, fichotheque, SubsetKey.CATEGORY_SPHERE))
                .putParameter(PiocheParameters.DEFAULTSPHEREKEY, getDefaultSphereKey(requestMap));
    }

    private static int getLimit(RequestMap requestMap) {
        String limitString = requestMap.getParameter(PiocheDomain.LIMIT_PARAMNAME);
        int limit = 5;
        if (limitString != null) {
            try {
                limit = Integer.parseInt(limitString);
                if (limit < 1) {
                    limit = 5;
                }
            } catch (NumberFormatException nfe) {
            }
        }
        return limit;
    }

    public static SubsetKey getDefaultSphereKey(RequestMap requestMap) {
        String param = requestMap.getParameter(PiocheDomain.PARAM_PARAMNAME);
        if (param == null) {
            return null;
        }
        try {
            SubsetKey sphereKey = SubsetKey.parse(param);
            if (sphereKey.isSphereSubset()) {
                return sphereKey;
            } else {
                return null;
            }
        } catch (java.text.ParseException pe) {
            return null;
        }
    }

    public static List<Subset> getSubsetList(RequestMap requestMap, Fichotheque fichotheque, short category) throws BdfInstructionException {
        String subsetsParam = BdfInstructionUtils.getMandatoryParameter(requestMap, PiocheDomain.SUBSETS_PARAMNAME);
        if (subsetsParam.length() == 0) {
            return null;
        } else {
            String[] array = StringUtils.getTechnicalTokens(subsetsParam, true);
            ArrayList<Subset> subsetList = new ArrayList<Subset>();
            for (String name : array) {
                try {
                    SubsetKey subsetKey = SubsetKey.parse(category, name);
                    Subset subset = fichotheque.getSubset(subsetKey);
                    if (subset != null) {
                        subsetList.add(subset);
                    }
                } catch (ParseException pe) {

                }
            }
            if (subsetList.isEmpty()) {
                throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.SUBSETS_PARAMNAME, subsetsParam));
            }
            return subsetList;
        }
    }

    public static SubsetKey getCroisementSubsetKey(RequestMap requestMap) throws BdfInstructionException {
        String subsetString = requestMap.getParameter(PiocheDomain.CROISEMENT_PARAMNAME);
        if (subsetString == null) {
            return null;
        }
        subsetString = subsetString.trim();
        if (subsetString.length() == 0) {
            return null;
        }
        try {
            SubsetKey subsetKey = SubsetKey.parse(subsetString);
            return subsetKey;
        } catch (ParseException pe) {
            throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(PiocheDomain.CROISEMENT_PARAMNAME, subsetString));
        }
    }

}
