/* BdfServer_HtmlProducers - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.sphere;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.SphereDomain;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class SphereHtmlProducerFactory {

    private final static int SPHERE_ADMIN = 1;
    private final static int REDACTEUR_ADMIN = 2;

    private SphereHtmlProducerFactory() {
    }

    public static HtmlProducer getHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        BdfServerHtmlProducer htmlProducer = getBdfHtmlProducer(parameters);
        if (htmlProducer != null) {
            htmlProducer.setBdfCommandResult(parameters.getBdfCommandResult());
        }
        return htmlProducer;
    }

    public static BdfServerHtmlProducer getBdfHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        String page = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        switch (getPageType(page)) {
            case SPHERE_ADMIN: {
                Sphere sphere = getSphere(bdfServer, requestMap, bdfCommandResult);
                parameters.checkSubsetAdmin(sphere);
                switch (page) {
                    case SphereDomain.SPHERE_ADVANCEDCOMMANDS_PAGE: {
                        return new SphereAdvancedCommandsHtmlProducer(parameters, sphere);
                    }
                    case SphereDomain.SPHERE_METADATAFORM_PAGE: {
                        return new SphereMetadataFormHtmlProducer(parameters, sphere);
                    }
                    case SphereDomain.REDACTEUR_CREATIONFORM_PAGE: {
                        return new RedacteurCreationFormHtmlProducer(parameters, sphere);
                    }
                    default:
                        return null;
                }
            }
            case REDACTEUR_ADMIN: {
                Redacteur redacteur = getRedacteur(bdfServer, requestMap, bdfCommandResult);
                parameters.checkSubsetAdmin(redacteur.getSphere());
                switch (page) {
                    case SphereDomain.REDACTEUR_ADVANCEDCOMMANDS_PAGE: {
                        return new RedacteurAvancedCommandsHtmlProducer(parameters, redacteur);
                    }
                    case SphereDomain.REDACTEUR_ADMINFORM_PAGE: {
                        return new RedacteurAdminFormHtmlProducer(parameters, redacteur);
                    }
                    case SphereDomain.REDACTEUR_STATE_PAGE: {
                        return new RedacteurStateHtmlProducer(parameters, redacteur);
                    }
                    default:
                        return null;
                }
            }
            default:
                switch (page) {
                    case SphereDomain.REDACTEUR_USERFORM_PAGE: {
                        return new RedacteurUserFormHtmlProducer(parameters);
                    }
                    case SphereDomain.SPHERE_PAGE: {
                        Sphere sphere = getSphere(bdfServer, requestMap, bdfCommandResult);
                        parameters.checkSubsetAccess(sphere);
                        return new SphereHtmlProducer(parameters, sphere);
                    }
                    case SphereDomain.SPHERE_CREATIONFORM_PAGE: {
                        parameters.checkGlobalAdmin();
                        return new SphereCreationFormHtmlProducer(parameters);
                    }
                    default:
                        return null;
                }
        }
    }

    private static int getPageType(String page) {
        switch (page) {
            case SphereDomain.SPHERE_METADATAFORM_PAGE:
            case SphereDomain.SPHERE_ADVANCEDCOMMANDS_PAGE:
            case SphereDomain.REDACTEUR_CREATIONFORM_PAGE:
                return SPHERE_ADMIN;
            case SphereDomain.REDACTEUR_ADMINFORM_PAGE:
            case SphereDomain.REDACTEUR_STATE_PAGE:
            case SphereDomain.REDACTEUR_ADVANCEDCOMMANDS_PAGE:
                return REDACTEUR_ADMIN;
            default:
                return 0;
        }
    }

    private static Sphere getSphere(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        Sphere sphere = null;
        if (bdfCommandResult != null) {
            sphere = (Sphere) bdfCommandResult.getResultObject(BdfInstructionConstants.SPHERE_OBJ);
            if (sphere == null) {
                Redacteur redacteur = (Redacteur) bdfCommandResult.getResultObject(BdfInstructionConstants.REDACTEUR_OBJ);
                if (redacteur != null) {
                    sphere = redacteur.getSphere();
                }
            }
        }
        if (sphere == null) {
            sphere = BdfInstructionUtils.getMandatorySphere(bdfServer.getFichotheque(), requestMap);
        }
        return sphere;
    }

    private static Redacteur getRedacteur(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        Redacteur redacteur = null;
        if (bdfCommandResult != null) {
            redacteur = (Redacteur) bdfCommandResult.getResultObject(BdfInstructionConstants.REDACTEUR_OBJ);
        }
        if (redacteur == null) {
            redacteur = BdfInstructionUtils.getMandatoryRedacteur(bdfServer.getFichotheque(), requestMap);
        }
        return redacteur;
    }

}
