/* BdfServer_Multi - Copyright (c) 2015-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.multi;

import fr.exemole.bdfserver.api.storage.BdfStorageException;
import fr.exemole.bdfserver.api.storage.ResourceStorage;
import fr.exemole.bdfserver.conf.ConfUtils;
import fr.exemole.bdfserver.tools.BdfMessageLocalisationFactory;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.ini.IniParser;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.localisation.MessageLocalisationFactory;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.security.PasswordChecker;


/**
 *
 * @author Vincent Calame
 */
public class MultiConf {

    public final static String MISSING_FILE_ERROR = "_ error.empty.inifile";
    public final static String LOGIN_PARAM = "login";
    public final static String PASSWORD_PARAM = "password";
    public final static String LANG_PARAM = "lang";
    public final static String AUTHORITY_PARAM = "authority";
    public final static String DEFAULT_FICHOTHEQUE_PARAM = "default_fichotheque";
    public final static short CONNECTED_STATE = 1;
    public final static short AUTHREQUIRED_STATE = 2;
    public final static short AUTHFAILED_STATE = 3;
    private final File iniFile;
    private final MessageLocalisationFactory messageLocalisationFactory;
    private final Lang defaultLang;
    private CommandMessage initErrorMessage;
    private String login;
    private String passwordHash;
    private Lang lang;
    private String authority;
    private String defaultFichothequeName;
    private MessageLocalisation messageLocalisation;

    private MultiConf(File iniFile, MessageLocalisationFactory messageLocalisationFactory, Lang defaultLang) {
        this.iniFile = iniFile;
        this.messageLocalisationFactory = messageLocalisationFactory;
        this.defaultLang = defaultLang;
        this.messageLocalisation = messageLocalisationFactory.newInstance(defaultLang);
        this.lang = defaultLang;
    }

    public void update() {
        if (iniFile.exists()) {
            Map<String, String> map = new HashMap<String, String>();
            try (InputStream is = new FileInputStream(iniFile)) {
                IniParser.parseIni(is, map);
            } catch (IOException ioe) {
                throw new BdfStorageException(ioe);
            }
            try {
                checkLogin(map);
                checkPassword(map);
                checkLang(map);
                checkAuthority(map);
                checkDefaultFichothequeName(map);
                initErrorMessage = null;
            } catch (ErrorMessageException eme) {
                initErrorMessage = eme.getErrorMessage();
            }
        } else {
            initErrorMessage = LogUtils.error(MISSING_FILE_ERROR, iniFile.getName());
        }
    }

    private void checkLogin(Map<String, String> map) throws ErrorMessageException {
        String value = map.get(LOGIN_PARAM);
        if (value != null) {
            value = value.trim();
            if (!value.isEmpty()) {
                this.login = value;
                return;
            }
        }
        throw new ErrorMessageException("_ error.empty.mandatoryparameter", LOGIN_PARAM);
    }

    private void checkLang(Map<String, String> map) throws ErrorMessageException {
        this.lang = defaultLang;
        String value = map.get(LANG_PARAM);
        if (value != null) {
            value = value.trim();
            if (!value.isEmpty()) {
                try {
                    this.lang = Lang.parse(value);
                } catch (ParseException pe) {
                    throw new ErrorMessageException("_ error.wrong.parametervalue", LANG_PARAM, value);
                }
            }
        }
        this.messageLocalisation = messageLocalisationFactory.newInstance(lang);
    }

    private boolean checkPassword(Map<String, String> map) throws ErrorMessageException {
        String value = map.get(PASSWORD_PARAM);
        if (value != null) {
            value = value.trim();
            if (!value.isEmpty()) {
                this.passwordHash = value;
                return false;
            }
        }
        throw new ErrorMessageException("_ error.empty.mandatoryparameter", PASSWORD_PARAM);
    }

    private void checkDefaultFichothequeName(Map<String, String> map) throws ErrorMessageException {
        String value = map.get(DEFAULT_FICHOTHEQUE_PARAM);
        if (value != null) {
            value = value.trim();
            if (!value.isEmpty()) {
                if (ConfUtils.isValidMultiName(value)) {
                    this.defaultFichothequeName = value;
                } else {
                    throw new ErrorMessageException("_ error.wrong.parametervalue", DEFAULT_FICHOTHEQUE_PARAM, value);
                }
            }
        }
    }

    private void checkAuthority(Map<String, String> map) throws ErrorMessageException {
        String value = map.get(AUTHORITY_PARAM);
        if (value == null) {
            value = map.get("uuid");
        }
        if (value != null) {
            value = value.trim();
            if (!value.isEmpty()) {
                this.authority = value;
                return;
            }
        }
        throw new ErrorMessageException("_ error.empty.mandatoryparameter", AUTHORITY_PARAM);
    }

    public File getInitFile() {
        return iniFile;
    }

    public CommandMessage getInitErrorMessage() {
        return initErrorMessage;
    }

    public Lang getLang() {
        return lang;
    }

    public String getAuthority() {
        return authority;
    }

    public MessageLocalisation getMessageLocalisation() {
        return messageLocalisation;
    }

    public MessageLocalisation getMessageLocalisation(Lang lang) {
        return messageLocalisationFactory.newInstance(lang);
    }

    public String getDefaultFichothequeName() {
        return defaultFichothequeName;
    }

    public short checkAuthentification(RequestMap requestMap) {
        if (initErrorMessage != null) {
            throw new IllegalStateException("multiAdmin has init error");
        }
        String loginValue = requestMap.getParameter(LOGIN_PARAM);
        String passwordValue = requestMap.getParameter(PASSWORD_PARAM);
        if ((loginValue == null) || (passwordValue == null)) {
            return AUTHREQUIRED_STATE;
        }
        if (!loginValue.equals(login)) {
            return AUTHFAILED_STATE;
        }
        if (!PasswordChecker.check(passwordValue, passwordHash)) {
            return AUTHFAILED_STATE;
        }
        return CONNECTED_STATE;
    }

    public static MultiConf build(File iniFile, List<ResourceStorage> webappsResourceStorageList) {
        BdfMessageLocalisationFactory messageLocalisationFactory = BdfMessageLocalisationFactory.buildFactory(webappsResourceStorageList);
        MultiConf multiConf = new MultiConf(iniFile, messageLocalisationFactory, Lang.getDefault());
        multiConf.update();
        return multiConf;
    }


}
