/* FichothequeLib_API - Copyright (c) 2015-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils;

import java.text.Collator;
import java.util.Comparator;
import net.fichotheque.Subset;
import net.fichotheque.SubsetItem;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.croisement.Liaison;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.thesaurus.Motcle;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.text.Label;


/**
 *
 * @author Vincent Calame
 */
public final class Comparators {

    public final static Comparator<Motcle> IDALPHA = new IdalphaComparator();
    public final static Comparator<FicheMeta> FICHEID = new FicheIdComparator();
    public final static Comparator<FicheMeta> CREATIONDATE = new CreationDateComparator();
    public final static Comparator<FicheMeta> MODIFICATIONDATE = new ModificationDateComparator();
    public final static Comparator<Liaison> LIAISONID = new LiaisonIdComparator();
    public final static Comparator<FieldKey> FIELDKEY = new FieldKeyComparator();
    public final static Comparator<Subset> SIZE = new SizeComparator();
    public final static Comparator<Redacteur> REDACTEUR = new RedacteurComparator();

    private Comparators() {

    }

    public static Comparator<FicheMeta> titre(Collator collator) {
        return new TitreComparator(collator);
    }

    public static Comparator<Motcle> label(Lang lang) {
        Collator collator = Collator.getInstance(lang.toLocale());
        collator.setStrength(Collator.PRIMARY);
        return new LabelComparator(lang, collator);
    }

    public static Comparator<Motcle> label(Lang lang, Collator collator) {
        return new LabelComparator(lang, collator);
    }

    public static Comparator<Motcle> idalphaAndLabel(Lang lang) {
        return new IdalphaAndLabelComparator(lang);
    }

    private static int compareId(SubsetItem piece1, SubsetItem piece2) {
        int result = piece1.getSubsetKey().compareTo(piece2.getSubsetKey());
        if (result != 0) {
            return result;
        }
        int id1 = piece1.getId();
        int id2 = piece2.getId();
        if (id1 < id2) {
            return -1;
        }
        if (id1 > id2) {
            return 1;
        }
        return 0;
    }


    private static int getWeight(FieldKey fieldKey) {
        switch (fieldKey.getCategory()) {
            case FieldKey.PROPRIETE_CATEGORY:
                return 30;
            case FieldKey.INFORMATION_CATEGORY:
                return 20;
            case FieldKey.SECTION_CATEGORY:
                return 10;
            case FieldKey.SPECIAL_CATEGORY:
                switch (fieldKey.getFieldString()) {
                    case FieldKey.SPECIAL_ID:
                        return 150;
                    case FieldKey.SPECIAL_TITRE:
                        return 140;
                    case FieldKey.SPECIAL_SOUSTITRE:
                        return 130;
                    case FieldKey.SPECIAL_LANG:
                        return 120;
                    case FieldKey.SPECIAL_REDACTEURS:
                        return 110;
                    default:
                        throw new SwitchException("Unknown special field = " + fieldKey.getFieldString());
                }
            default:
                throw new SwitchException("Unknown category = " + fieldKey.getCategory());
        }
    }


    private static class FicheIdComparator implements Comparator<FicheMeta> {

        private FicheIdComparator() {

        }

        @Override
        public int compare(FicheMeta ficheMeta1, FicheMeta ficheMeta2) {
            return compareId(ficheMeta1, ficheMeta2);
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof FicheIdComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1001;
        }

    }


    private static class IdalphaComparator implements Comparator<Motcle> {

        private IdalphaComparator() {
        }

        @Override
        public int compare(Motcle motcle1, Motcle motcle2) {
            String idalpha1 = motcle1.getIdalpha();
            String idalpha2 = motcle2.getIdalpha();
            return idalpha1.compareTo(idalpha2);
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof IdalphaComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1002;
        }

    }


    private static class CreationDateComparator implements Comparator<FicheMeta> {

        private CreationDateComparator() {
        }

        @Override
        public int compare(FicheMeta ficheMeta1, FicheMeta ficheMeta2) {
            int resultat = -FuzzyDate.compare(ficheMeta1.getCreationDate(), ficheMeta2.getCreationDate());
            if (resultat == 0) {
                resultat = -compareId(ficheMeta1, ficheMeta2);
            }
            return resultat;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof CreationDateComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1003;
        }

    }


    private static class ModificationDateComparator implements Comparator<FicheMeta> {

        private ModificationDateComparator() {
        }

        @Override
        public int compare(FicheMeta ficheMeta1, FicheMeta ficheMeta2) {
            int resultat = -FuzzyDate.compare(ficheMeta1.getModificationDate(), ficheMeta2.getModificationDate());
            if (resultat == 0) {
                resultat = -FuzzyDate.compare(ficheMeta1.getCreationDate(), ficheMeta2.getCreationDate());
                if (resultat == 0) {
                    resultat = -compareId(ficheMeta1, ficheMeta2);
                }
            }
            return resultat;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof ModificationDateComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1004;
        }

    }


    private static class TitreComparator implements Comparator<FicheMeta> {

        private final Collator collator;

        private TitreComparator(Collator collator) {
            this.collator = collator;
        }

        @Override
        public int compare(FicheMeta ficheMeta1, FicheMeta ficheMeta2) {
            int resultat = collator.compare(ficheMeta1.getTitre(), ficheMeta2.getTitre());
            if (resultat == 0) {
                resultat = compareId(ficheMeta1, ficheMeta2);
            }
            return resultat;
        }

    }


    private static class LabelComparator implements Comparator<Motcle> {

        private final Lang lang;
        private final Collator collator;

        public LabelComparator(Lang lang, Collator collator) {
            this.lang = lang;
            this.collator = collator;
        }

        @Override
        public int compare(Motcle motcle1, Motcle motcle2) {
            int resultat = collator.compare(toComparableString(motcle1), toComparableString(motcle2));
            if (resultat == 0) {
                resultat = compareId(motcle1, motcle2);
            }
            return resultat;
        }

        private String toComparableString(Motcle motcle) {
            Label label = motcle.getLabels().getLabel(lang);
            if (label != null) {
                return label.getLabelString();
            } else {
                return "";
            }
        }

    }


    private static class IdalphaAndLabelComparator implements Comparator<Motcle> {

        private final Lang lang;

        public IdalphaAndLabelComparator(Lang lang) {
            this.lang = lang;
        }

        @Override
        public int compare(Motcle mc1, Motcle mc2) {
            String lib1 = toComparableString(mc1);
            String lib2 = toComparableString(mc2);
            int result;
            result = lib1.compareTo(lib2);
            if (result != 0) {
                return result;
            }
            result = mc1.getSubsetKey().compareTo(mc2.getSubsetKey());
            if (result != 0) {
                return result;
            }
            int id1 = mc1.getId();
            int id2 = mc2.getId();
            if (id1 < id2) {
                return -1;
            }
            if (id1 > id2) {
                return 1;
            }
            return 0;
        }

        private String toComparableString(Motcle motcle) {
            String idalpha = motcle.getIdalpha();
            Label label = motcle.getLabels().getLabel(lang);
            if (idalpha != null) {
                if (label != null) {
                    return idalpha + " – " + label.getLabelString();
                } else {
                    return idalpha;
                }
            } else {
                if (label != null) {
                    return label.getLabelString();
                } else {
                    return "";
                }
            }
        }

    }


    private static class FieldKeyComparator implements Comparator<FieldKey> {

        private FieldKeyComparator() {
        }

        @Override
        public int compare(FieldKey fieldKey1, FieldKey fieldKey2) {
            int weight1 = getWeight(fieldKey1);
            int weight2 = getWeight(fieldKey2);
            if (weight1 > weight2) {
                return -1;
            } else if (weight1 < weight2) {
                return 1;
            }
            if (weight1 > 100) {
                return 0;
            }
            return fieldKey1.getFieldName().compareTo(fieldKey2.getFieldName());
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof FieldKeyComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1005;
        }

    }


    private static class LiaisonIdComparator implements Comparator<Liaison> {

        private LiaisonIdComparator() {

        }

        @Override
        public int compare(Liaison liaison1, Liaison liaison2) {
            return compareId(liaison1.getSubsetItem(), liaison2.getSubsetItem());
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof FicheIdComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1006;
        }

    }


    private static class SizeComparator implements Comparator<Subset> {

        private SizeComparator() {

        }

        @Override
        public int compare(Subset subset1, Subset subset2) {
            int size1 = subset1.size();
            int size2 = subset2.size();
            if (size1 > size2) {
                return -1;
            }
            if (size1 < size2) {
                return 1;
            }
            return subset1.getSubsetKey().compareTo(subset2.getSubsetKey());
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof SizeComparator) {
                return true;
            }
            return false;
        }

        @Override
        public int hashCode() {
            return 1007;
        }

    }


    private static class RedacteurComparator implements Comparator<Redacteur> {

        private RedacteurComparator() {
        }

        @Override
        public int compare(Redacteur r1, Redacteur r2) {
            String id1 = r1.getLogin();
            String id2 = r2.getLogin();
            int result = id1.compareTo(id2);
            if (result != 0) {
                return result;
            }
            String nc1 = r1.getPersonCore().toDirectoryStyle(true);
            String nc2 = r2.getPersonCore().toDirectoryStyle(true);
            result = nc1.compareTo(nc2);
            if (result != 0) {
                return result;
            }
            return r1.getSubsetKey().compareTo(r2.getSubsetKey());
        }

    }

}
